from .baseobject import BareObject
from .baseobject import Element
from .baseobject import PropertyType


class Container(Element):
    """
        "type": {
          "enum": [
            "Container"
          ],
          "description": "Must be `Container`"
        },
        "items": {
          "type": "array",
          "items": {
            "$ref": "#/definitions/ImplementationsOf.Element"
          },
          "description": "The card elements to render inside the `Container`."
        },
        "selectAction": {
          "$ref": "#/definitions/ImplementationsOf.ISelectAction",
          "description": "An Action that will be invoked when the `Container` is tapped or selected. `Action.ShowCard` is not supported.",
          "version": "1.1"
        },
        "style": {
          "anyOf": [
            {
              "$ref": "#/definitions/ContainerStyle"
            },
            {
              "type": "null"
            }
          ],
          "description": "Style hint for `Container`."
        },
        "verticalContentAlignment": {
          "$ref": "#/definitions/VerticalContentAlignment",
          "description": "Defines how the content should be aligned vertically within the container.",
          "default": "top",
          "version": "1.1"
        },
        "bleed": {
          "type": "boolean",
          "description": "Determines whether the element should bleed through its parent's padding.",
          "version": "1.2",
          "features": [
            2109
          ]
        },
        "backgroundImage": {
          "anyOf": [
            {
              "$ref": "#/definitions/BackgroundImage"
            },
            {
              "type": "string",
              "format": "uri-reference",
              "description": "The URL (or data url) to use as the background image. Supports data URI."
            }
          ],
          "description": "Specifies the background image. Acceptable formats are PNG, JPEG, and GIF",
          "version": "1.2"
        },
        "minHeight": {
          "type": "string",
          "description": "Specifies the minimum height of the container in pixels, like `\"80px\"`.",
          "examples": [
            "50px"
          ],
          "version": "1.2",
          "features": [
            2293
          ]
        },
        "fallback": {},
        "height": {},
        "separator": {},
        "spacing": {},
        "id": {},
        "isVisible": {},
        "requires": {}
    """

    element_type = 'Container'
    bleed = PropertyType(type=bool)
    min_height = PropertyType(type=str, key_name='minHeight')
    background_image = PropertyType(type=str, key_name='backgroundImage')
    vertical_content_alignment = PropertyType(type=str, key_name='verticalContentAlignment')
    style = PropertyType(type=str)
    select_action = PropertyType(type=str, key_name='selectAction')
    items = PropertyType(type=list)


# TODO: Correct `actions` as array of Action
class ActionSet(Element):
    element_type = 'ActionSet'
    actions = PropertyType(type=list)


# TODO: Correct `columns` as array of Column
class ColumnSet(Element):
    element_type = 'ColumnSet'
    columns = PropertyType(type=list)
    select_action = PropertyType(type=str, key_name='selectAction')
    style = PropertyType(type=str)
    bleed = PropertyType(type=bool)
    min_height = PropertyType(type=str, key_name='minHeight')
    horizontal_alignment = PropertyType(type=str, key_name='horizontalAlignment')


class Column(Element):
    element_type = 'Column'
    items = PropertyType(type=list, key_name='items')
    backgroundImage = PropertyType(type=str, key_name='backgroundImage')
    bleed = PropertyType(type=bool, key_name='bleed')
    minHeight = PropertyType(type=str, key_name='minHeight')
    horizontalAlignment = PropertyType(type=str, key_name='horizontalAlignment')
    separator = PropertyType(type=bool, key_name='separator')
    selectAction = PropertyType(type=str, key_name='selectAction')
    style = PropertyType(type=str, key_name='style')
    verticalContentAlignment = PropertyType(type=str, key_name='verticalContentAlignment')
    width = PropertyType(type=str, key_name='width')


class FactSet(Element):
    element_type = 'FactSet'
    facts = PropertyType(type=list, key_name='facts')


class Fact(BareObject):
    title = PropertyType(type=str)
    value = PropertyType(type=str)


# TODO: Correct `images` as array of Image
class ImageSet(Element):
    element_type = 'ImageSet'
    images = PropertyType(type=list)
    image_size = PropertyType(type=str, key_name='imageSize')



import json


class PropertyType:
    def __init__(self, type, key_name=None):
        self.key_name = key_name
        self.type = type

    def __get__(self, instance, owner):
        return instance.get_data(self.key_name)

    def __set__(self, instance, value):
        if not isinstance(value, self.type):
            raise TypeError(f'{self.key_name} must be of type(s) {self.type} (got {value!r})')
        instance.set_data(self.key_name, value)

    def __set_name__(self, owner, name):
        if self.key_name is None:
            self.key_name = name


class AdaptiveCardJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, BareObject):
            return obj._data

        # Let the base class default method raise the TypeError
        return json.JSONEncoder.default(self, obj)



class BareObject():
    def __init__(self, *args, **kwargs):
        super().__init__()
        self._data = {}

        # init data
        if kwargs:
            for prop in tuple(kwargs):
                try:
                    self.__setattr__(prop, kwargs[prop])
                except AttributeError:
                    pass

    def __str__(self):
        return json.dumps(self.render(), sort_keys=True, indent=2, cls=AdaptiveCardJSONEncoder)

    def __repr__(self):
        return self.__str__()

    def render(self):
        """
        Transform instance object to dict that follows the schema of adaptivecards.io
        Returns
        -------
        dict : data follow schema of adaptivecards
        """
        return self._data

    def get_data(self, key):
        return self._data.get(key)

    def set_data(self, key, value):
        self._data[key] = value


class Element(BareObject):
    element_type = 'None'
    fallback = PropertyType(type=str)
    height = PropertyType(type=str)
    separator = PropertyType(type=bool)
    spacing = PropertyType(type=str)
    visible = PropertyType(key_name='isVisible', type=bool)
    id = PropertyType(type=str)
    requires = PropertyType(type=dict)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._data['type'] = self.element_type



from .baseobject import BareObject
from .baseobject import PropertyType


class AdaptiveCard(BareObject):
    """
    Reference document for schema: https://adaptivecards.io/explorer/

    An Adaptive Card, containing a free-form body of card elements, and an optional set of actions.

    Property
    --------
    type : "AdaptiveCard"
        Must be "AdaptiveCard".	
    version : string
        Schema version that this card requires. If a client is lower than this version,
        the fallbackText will be rendered.
        NOTE: Version is not required for cards within an Action.ShowCard.
        However, it is required for the top-level card.
    refresh	: Refresh
        Defines how the card can be refreshed by making a request to the target Bot.
    authentication : Authentication
        Defines authentication information to enable on-behalf-of single sign on or just-in-time OAuth.
    body : Element[]
        The card elements to show in the primary card region.
    actions : Action[]
        The Actions to show in the card’s action bar.
    selectAction : ISelectAction
        An Action that will be invoked when the card is tapped or selected. Action.ShowCard is not supported.
    fallbackText : string
        Text shown when the client doesn’t support the version specified (may contain markdown).
    backgroundImage: BackgroundImage, uri
        Specifies the background image of the card.
    minHeight : string
        Specifies the minimum height of the card.
    speak : string
        Specifies what should be spoken for this entire card. This is simple text or SSML fragment.
    lang : string
        The 2-letter ISO-639-1 language used in the card. Used to localize any date/time functions.
    verticalContentAlignment :VerticalContentAlignment
        Defines how the content should be aligned vertically within the container.
        Only relevant for fixed-height cards, or cards with a minHeight specified.
    $schema : uri
        The Adaptive Card schema.
    """
    background_image = PropertyType(type=str, key_name='backgroundImage')
    vertical_content_alignment = PropertyType(type=str, key_name='verticalContentAlignment')
    select_action = PropertyType(type=str, key_name='selectAction')
    min_height = PropertyType(type=str, key_name='minHeight')
    fallback_text = PropertyType(type=str, key_name='fallbackText')
    body = PropertyType(type=list, key_name='body')
    actions = PropertyType(type=list, key_name='actions')
    speak = PropertyType(type=str, key_name='speak')
    lang = PropertyType(type=str, key_name='lang')

    def __init__(self) -> None:
        super().__init__()
        self.set_data('type', 'AdaptiveCard')
        self.set_data('version', '1.2')
        self.set_data('$schema', 'http://adaptivecards.io/schemas/adaptive-card.json')



class TeamsAdaptiveMessage(BareObject):
    element_type = PropertyType(type=str, key_name='type')
    attachments = PropertyType(type=list)

    def __init__(self, card: AdaptiveCard):
        super(TeamsAdaptiveMessage, self).__init__()
        self.element_type = 'message'
        self.attachments = [{
            "contentType": "application/vnd.microsoft.card.adaptive",
            "contentUrl": None,
            "content": card
        }]



from .baseobject import BareObject
from .baseobject import Element
from .baseobject import PropertyType


class TextBlock(Element):
    element_type = 'TextBlock'
    text = PropertyType(type=str, key_name='text')
    font_type = PropertyType(type=str, key_name='fontType')
    color = PropertyType(type=str, key_name='color')
    horizontalAlignment = PropertyType(type=str, key_name='horizontalAlignment')
    isSubtle = PropertyType(type=bool, key_name='isSubtle')
    maxLines = PropertyType(type=int, key_name='maxLines')
    size = PropertyType(type=str, key_name='size')
    weight = PropertyType(type=str, key_name='weight')
    wrap = PropertyType(type=bool, key_name='wrap')


class Image(Element):
    element_type = 'Image'
    url = PropertyType(type=str, key_name='url')
    altText = PropertyType(type=str, key_name='altText')
    backgroundColor = PropertyType(type=str, key_name='backgroundColor')
    height = PropertyType(type=str, key_name='height')
    horizontalAlignment = PropertyType(type=str, key_name='horizontalAlignment')
    selectAction = PropertyType(type=str, key_name='selectAction')
    size = PropertyType(type=str, key_name='size')
    style = PropertyType(type=str, key_name='style')
    width = PropertyType(type=str, key_name='width')


# TODO: Correct type for sources as array of MediaSource
class Media(Element):
    element_type = 'Media'
    sources = PropertyType(type=list, key_name='sources')
    poster = PropertyType(type=str, key_name='poster')
    altText = PropertyType(type=str, key_name='altText')


class MediaSource(BareObject):
    mimeType = PropertyType('mimeType')
    url = PropertyType(type=str, key_name='url')


"""
Example for RichTextBlock:
{
  "$schema": "http://adaptivecards.io/schemas/adaptive-card.json",
  "type": "AdaptiveCard",
  "version": "1.2",
  "body": [
    {
      "type": "RichTextBlock",
      "inlines": [
        "This is the first inline. ",
        {
          "type": "TextRun",
          "text": "We support colors,",
          "color": "good"
        },
        {
          "type": "TextRun",
          "text": " both regular and subtle. ",
          "isSubtle": true
        },
        {
          "type": "TextRun",
          "text": "Text ",
          "size": "small"
        },
        {
          "type": "TextRun",
          "text": "of ",
          "size": "medium"
        },
        {
          "type": "TextRun",
          "text": "all ",
          "size": "large"
        },
        {
          "type": "TextRun",
          "text": "sizes! ",
          "size": "extraLarge"
        },
        {
          "type": "TextRun",
          "text": "Light weight text. ",
          "weight": "lighter"
        },
        {
          "type": "TextRun",
          "text": "Bold weight text. ",
          "weight": "bolder"
        },
        {
          "type": "TextRun",
          "text": "Highlights. ",
          "highlight": true
        },
        {
          "type": "TextRun",
          "text": "Italics. ",
          "italic": true
        },
        {
          "type": "TextRun",
          "text": "Strikethrough. ",
          "strikethrough": true
        },
        {
          "type": "TextRun",
          "text": "Monospace too!",
          "fontType": "monospace"
        }
      ]
    },
    {
      "type": "RichTextBlock",
      "inlines": [
        {
          "type": "TextRun",
          "text": "Date-Time parsing: {{DATE(2017-02-14T06:08:39Z,LONG)}} {{TIME(2017-02-14T06:08:39Z)}}"
        }
      ]
    },
    {
      "type": "RichTextBlock",
      "horizontalAlignment": "center",
      "inlines": [
        {
          "type": "TextRun",
          "text": "Rich text blocks also support center alignment. Lorem ipsum dolor Lorem ipsum dolor Lorem ipsum dolor Lorem ipsum dolor Lorem ipsum dolor "
        }
      ]
    },
    {
      "type": "RichTextBlock",
      "horizontalAlignment": "right",
      "inlines": [
        {
          "type": "TextRun",
          "text": "Rich text blocks also support right alignment. Lorem ipsum dolor Lorem ipsum dolor Lorem ipsum dolor Lorem ipsum dolor Lorem ipsum dolor "
        }
      ]
    },
    {
      "type": "RichTextBlock",
      "inlines": []
    }
  ]
}
"""


class RichTextBlock(Element):
    element_type = 'RichTextBlock'
    url = PropertyType(type=str, key_name='url')
    inlines = PropertyType(type=list, key_name='inlines')
    horizontalAlignment = PropertyType(type=str, key_name='horizontalAlignment')


class TextRun(Element):
    element_type = 'TextRun'
    text = PropertyType(type=str, key_name='text')
    color = PropertyType(type=str, key_name='color')
    fontType = PropertyType(type=str, key_name='fontType')
    highlight = PropertyType(type=bool, key_name='highlight')
    isSubtle = PropertyType(type=bool, key_name='isSubtle')
    italic = PropertyType(type=bool, key_name='italic')
    selectAction = PropertyType(type=str, key_name='selectAction')
    size = PropertyType(type=str, key_name='size')
    strikethrough = PropertyType(type=bool, key_name='strikethrough')
    underline = PropertyType(type=bool, key_name='underline')
    weight = PropertyType(type=str, key_name='weight')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

requirements = [ ]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="Huu Hoa NGUYEN",
    author_email='huuhoa@gmail.com',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="Author adaptive cards in pure python",
    install_requires=requirements,
    license="MIT License",
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='adaptivecards',
    name='adaptivecards',
    packages=find_packages(include=['adaptivecards']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/huuhoa/adaptivecards',
    version='0.4.0',
    zip_safe=False,
)

